@@ -1,13 +1,13 @@
 import abc
 import logging
-import urlparse
 import datetime
 import random
 
 import yaml
+from six.moves.urllib.parse import urlparse
 
 import flask
-from flask import request, session
+from flask import current_app, request, session
 from flask import abort, jsonify, redirect
 from werkzeug.security import safe_str_cmp
 
@@ -18,20 +18,21 @@ from authomatic.adapters import WerkzeugAdapter
 
 # saml auth imports
 from onelogin.saml2.auth import OneLogin_Saml2_Auth
+
+from confidant import settings
 from confidant.lib import cryptolib
 from confidant.utils.misc import dict_deep_update
-
-from confidant.app import app
-
 from confidant.authnz import errors
 
+logger = logging.getLogger(__name__)
+
 
 def init_user_auth_class(*args, **kwargs):
-    if not app.config['USE_AUTH']:
+    if not settings.USE_AUTH:
         module = NullUserAuthenticator
 
     else:
-        module_name = app.config['USER_AUTH_MODULE'].lower()
+        module_name = settings.USER_AUTH_MODULE.lower()
         if module_name == 'google':
             module = GoogleOauthAuthenticator
         elif module_name == 'saml':
@@ -45,7 +46,7 @@ def init_user_auth_class(*args, **kwargs):
                 'Unknown USER_AUTH_MODULE: {!r}'.format(module_name))
 
     auth = module(*args, **kwargs)
-    logging.info('Initializing {} user authenticator'.format(auth.auth_type))
+    logger.info('Initializing {} user authenticator'.format(auth.auth_type))
     return auth
 
 
@@ -64,7 +65,7 @@ class AbstractUserAuthenticator(object):
         if 'expiration' in session:
             # Paranoia case
             if session.get('max_expiration') is None:
-                logging.warning(
+                logger.warning(
                     'max_expiration unset on session, when expiration is set.'
                 )
                 return True
@@ -80,10 +81,10 @@ class AbstractUserAuthenticator(object):
         return session['user']
 
     def get_csrf_token(self):
-        return session.get(app.config['XSRF_COOKIE_NAME'])
+        return session.get(settings.XSRF_COOKIE_NAME)
 
     def set_csrf_token(self, resp):
-        cookie_name = app.config['XSRF_COOKIE_NAME']
+        cookie_name = settings.XSRF_COOKIE_NAME
         if cookie_name not in session:
             session[cookie_name] = '{0:x}'.format(
                 random.SystemRandom().getrandbits(160)
@@ -91,23 +92,23 @@ class AbstractUserAuthenticator(object):
         resp.set_cookie(cookie_name, session[cookie_name])
 
     def check_csrf_token(self):
-        cookie_name = app.config['XSRF_COOKIE_NAME']
+        cookie_name = settings.XSRF_COOKIE_NAME
         token = request.headers.get('X-XSRF-TOKEN', '')
         if not token:
             return False
         return safe_str_cmp(token, session.get(cookie_name, ''))
 
     def set_expiration(self):
-        if app.config['PERMANENT_SESSION_LIFETIME']:
+        if settings.PERMANENT_SESSION_LIFETIME:
             session.permanent = True
             now = datetime.datetime.utcnow()
-            lifetime = app.config['PERMANENT_SESSION_LIFETIME']
+            lifetime = settings.PERMANENT_SESSION_LIFETIME
             expiration = now + datetime.timedelta(seconds=lifetime)
             session['expiration'] = expiration
             # We want the max_expiration initially set, but we don't want it to
             # be extended.
             if not session.get('max_expiration'):
-                max_lifetime = app.config['MAX_PERMANENT_SESSION_LIFETIME']
+                max_lifetime = settings.MAX_PERMANENT_SESSION_LIFETIME
                 if not max_lifetime:
                     max_lifetime = lifetime
                 max_expiration = now + datetime.timedelta(seconds=max_lifetime)
@@ -121,7 +122,9 @@ class AbstractUserAuthenticator(object):
         }
 
     def current_email(self):
-        return self.current_user()['email'].lower()
+        ret = self.current_user()['email'].lower()
+        # when migrating from 2 -> 3, the session email object may be bytes
+        return ret.decode('UTF-8') if isinstance(ret, bytes) else ret
 
     def current_first_name(self):
         return self.current_user()['first_name']
@@ -130,10 +133,10 @@ class AbstractUserAuthenticator(object):
         return self.current_user()['last_name']
 
     def redirect_to_index(self):
-        return redirect(flask.url_for('index'))
+        return redirect(flask.url_for('static_files.index'))
 
     def redirect_to_goodbye(self):
-        return redirect(flask.url_for('goodbye'))
+        return redirect(flask.url_for('static_files.goodbye'))
 
     @abc.abstractmethod
     def log_in(self):
@@ -167,13 +170,13 @@ class AbstractUserAuthenticator(object):
         clear_session(), then redirect to the desired post-logout page
         (e.g. with redirect_to_goodbye()).
         """
-        logging.info('Using default log_out() method')
+        logger.info('Using default log_out() method')
         self.clear_session()
 
         return self.redirect_to_goodbye()
 
     def clear_session(self):
-        logging.info('Clearing flask session')
+        logger.info('Clearing flask session')
         session['user'] = {}
         session.clear()
 
@@ -183,13 +186,14 @@ class AbstractUserAuthenticator(object):
         A whitelist of authorized email addresses or None.
         Loaded from config['USERS_FILE'] as YAML.
         """
-        if not hasattr(self, '_email_whitelist'):
-            self._email_whitelist = None
-            if app.config['USERS_FILE']:
-                with open(app.config['USERS_FILE'], 'r') as f:
-                    self._email_whitelist = yaml.safe_load(f.read())
+        # TODO: cache the _email_whitelist in memory, and check the file mtime
+        # to determine if the cache needs to be refreshed.
+        _email_whitelist = None
+        if settings.USERS_FILE:
+            with open(settings.USERS_FILE, 'r') as f:
+                _email_whitelist = yaml.safe_load(f.read())
 
-        return self._email_whitelist
+        return _email_whitelist
 
     @property
     def allowed_email_suffix(self):
@@ -200,7 +204,7 @@ class AbstractUserAuthenticator(object):
         Returns either a string or None.
         """
 
-        return app.config['USER_EMAIL_SUFFIX']
+        return settings.USER_EMAIL_SUFFIX
 
     def check_authorization(self):
         email = self.current_email()
@@ -211,7 +215,7 @@ class AbstractUserAuthenticator(object):
             raise errors.NotAuthorized(msg)
 
         if not self.passes_email_whitelist(email):
-            msg = 'User not in whitelist: {!r}'.format(
+            msg = 'User {!r} not in whitelist: {!r}'.format(
                 email, self.allowed_email_whitelist)
             raise errors.NotAuthorized(msg)
 
@@ -230,14 +234,14 @@ class AbstractUserAuthenticator(object):
             return True
 
 
-class NullUserAuthenticator(object):
+class NullUserAuthenticator(AbstractUserAuthenticator):
     """
     Fake user authenticator class that performs no authentication.
     """
 
     def __init__(self):
         # guard against using this when you think auth is in use
-        assert not app.config['USE_AUTH']
+        assert not settings.USE_AUTH
 
     @property
     def auth_type(self):
@@ -250,15 +254,6 @@ class NullUserAuthenticator(object):
             'last_name': 'user',
         }
 
-    def current_email(self):
-        return self.current_user()['email'].lower()
-
-    def current_first_name(self):
-        return self.current_user()['first_name']
-
-    def current_last_name(self):
-        return self.current_user()['last_name']
-
     def is_authenticated(self):
         """Null users are always authenticated"""
         return True
@@ -285,10 +280,10 @@ class HeaderAuthenticator(AbstractUserAuthenticator):
     """
 
     def __init__(self):
-        self.username_header = app.config['HEADER_AUTH_USERNAME_HEADER']
-        self.email_header = app.config['HEADER_AUTH_EMAIL_HEADER']
-        self.first_name_header = app.config.get('HEADER_AUTH_FIRST_NAME_HEADER')
-        self.last_name_header = app.config.get('HEADER_AUTH_LAST_NAME_HEADER')
+        self.username_header = settings.HEADER_AUTH_USERNAME_HEADER
+        self.email_header = settings.HEADER_AUTH_EMAIL_HEADER
+        self.first_name_header = settings.get('HEADER_AUTH_FIRST_NAME_HEADER')
+        self.last_name_header = settings.get('HEADER_AUTH_LAST_NAME_HEADER')
 
     @property
     def auth_type(self):
@@ -322,15 +317,6 @@ class HeaderAuthenticator(AbstractUserAuthenticator):
 
         return info
 
-    def current_email(self):
-        return self.current_user()['email'].lower()
-
-    def current_first_name(self):
-        return self.current_user()['first_name']
-
-    def current_last_name(self):
-        return self.current_user()['last_name']
-
     def is_authenticated(self):
         """Any user that is able to make requests is authenticated"""
         self.assert_headers()
@@ -364,8 +350,8 @@ class GoogleOauthAuthenticator(AbstractUserAuthenticator):
         self.authomatic_config = {
             'google': {
                 'class_': oauth2.Google,
-                'consumer_key': app.config['GOOGLE_OAUTH_CLIENT_ID'],
-                'consumer_secret': app.config['GOOGLE_OAUTH_CONSUMER_SECRET'],
+                'consumer_key': settings.GOOGLE_OAUTH_CLIENT_ID,
+                'consumer_secret': settings.GOOGLE_OAUTH_CONSUMER_SECRET,
                 'scope': [
                     'profile',
                     'email'
@@ -375,7 +361,7 @@ class GoogleOauthAuthenticator(AbstractUserAuthenticator):
 
         self.authomatic = Authomatic(
             self.authomatic_config,
-            app.config['AUTHOMATIC_SALT']
+            settings.AUTHOMATIC_SALT
         )
 
     @property
@@ -388,13 +374,13 @@ class GoogleOauthAuthenticator(AbstractUserAuthenticator):
             WerkzeugAdapter(request, response),
             'google',
             session=session,
-            session_saver=lambda: app.save_session(session, response),
+            session_saver=lambda: current_app.save_session(session, response),
             secure_cookie=(True if request.is_secure else False)
         )
         if result:
             if result.error:
                 msg = 'Google auth failed with error: {0}'
-                logging.error(msg.format(result.error.message))
+                logger.error(msg.format(result.error))
                 return abort(403)
 
             # successful login
@@ -440,15 +426,16 @@ class SamlAuthenticator(AbstractUserAuthenticator):
 
     def _render_saml_settings_dict(self):
         """
-        Given the configuration present in app.config, render a settings dict
-        suitable for passing to OneLogin_Saml2_Auth() in initialization.
+        Given the configuration present in current_app.config, render a
+        settings dict suitable for passing to OneLogin_Saml2_Auth() in
+        initialization.
         """
 
-        debug = app.config['SAML_DEBUG']
+        debug = settings.SAML_DEBUG
         if debug is None:
-            debug = app.debug
+            debug = current_app.debug
 
-        root_url = app.config['SAML_CONFIDANT_URL_ROOT']
+        root_url = settings.SAML_CONFIDANT_URL_ROOT
         if not root_url:
             raise ValueError("Must provide SAML_CONFIDANT_URL_ROOT")
         root_url = root_url.rstrip('/')
@@ -471,20 +458,20 @@ class SamlAuthenticator(AbstractUserAuthenticator):
         }
 
         sp_has_key = False
-        if app.config['SAML_SP_KEY_FILE']:
+        if settings.SAML_SP_KEY_FILE:
             sp_has_key = True
             sp_data['privateKey'] = self._load_rsa_for_saml(
-                app.config['SAML_SP_KEY_FILE'],
-                password=app.config.get('SAML_SP_KEY_FILE_PASSWORD'))
-        if app.config['SAML_SP_KEY']:
+                settings.SAML_SP_KEY_FILE,
+                password=settings.get('SAML_SP_KEY_FILE_PASSWORD'))
+        if settings.SAML_SP_KEY:
             sp_has_key = True
-            sp_data['privateKey'] = app.config['SAML_SP_KEY']
+            sp_data['privateKey'] = settings.SAML_SP_KEY
 
-        if app.config['SAML_SP_CERT_FILE']:
+        if settings.SAML_SP_CERT_FILE:
             sp_data['x509cert'] = self._load_x509_for_saml(
-                app.config['SAML_SP_CERT_FILE'])
-        if app.config['SAML_SP_CERT']:
-            sp_data['x509cert'] = app.config['SAML_SP_CERT']
+                settings.SAML_SP_CERT_FILE)
+        if settings.SAML_SP_CERT:
+            sp_data['x509cert'] = settings.SAML_SP_CERT
 
         # security defaults: sign everything if SP key was provided
         security_data = {
@@ -492,38 +479,38 @@ class SamlAuthenticator(AbstractUserAuthenticator):
             'authnRequestsSigned': sp_has_key,
             'logoutRequestsSigned': sp_has_key,
             'logoutResponsesSigned':
-                app.config['SAML_SECURITY_SLO_RESP_SIGNED'],
+                settings.SAML_SECURITY_SLO_RESP_SIGNED,
             'signMetadata': sp_has_key,
             'wantMessagesSigned':
-                app.config['SAML_SECURITY_MESSAGES_SIGNED'],
+                settings.SAML_SECURITY_MESSAGES_SIGNED,
             'wantAssertionsSigned':
-                app.config['SAML_SECURITY_ASSERTIONS_SIGNED'],
+                settings.SAML_SECURITY_ASSERTIONS_SIGNED,
             'wantNameIdEncrypted': False,
             'wantAttributeStatement':
-                app.config['SAML_WANT_ATTRIBUTE_STATEMENT'],
-            "signatureAlgorithm": app.config['SAML_SECURITY_SIG_ALGO'],
+                settings.SAML_WANT_ATTRIBUTE_STATEMENT,
+            "signatureAlgorithm": settings.SAML_SECURITY_SIG_ALGO,
         }
 
         # Identity provider section
         idp_data = {
-            'entityId': app.config['SAML_IDP_ENTITY_ID'],
+            'entityId': settings.SAML_IDP_ENTITY_ID,
             'singleSignOnService': {
-                'url': app.config['SAML_IDP_SIGNON_URL'],
+                'url': settings.SAML_IDP_SIGNON_URL,
                 'binding': 'urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect'
             },
         }
 
-        if app.config['SAML_IDP_LOGOUT_URL']:
+        if settings.SAML_IDP_LOGOUT_URL:
             idp_data['singleLogoutService'] = {
-                'url': app.config['SAML_IDP_LOGOUT_URL'],
+                'url': settings.SAML_IDP_LOGOUT_URL,
                 'binding': 'urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect'
             }
 
-        if app.config['SAML_IDP_CERT_FILE']:
+        if settings.SAML_IDP_CERT_FILE:
             idp_data['x509cert'] = self._load_x509_for_saml(
-                app.config['SAML_IDP_CERT_FILE'])
-        if app.config['SAML_IDP_CERT']:
-            idp_data['x509cert'] = app.config['SAML_IDP_CERT']
+                settings.SAML_IDP_CERT_FILE)
+        if settings.SAML_IDP_CERT:
+            idp_data['x509cert'] = settings.SAML_IDP_CERT
 
         # put it all together into the settings
         data = {
@@ -536,11 +523,11 @@ class SamlAuthenticator(AbstractUserAuthenticator):
 
         # if SAML_RAW_JSON_SETTINGS is set, merge the settings in, doing one
         # level of deep merging.
-        if app.config['SAML_RAW_JSON_SETTINGS']:
-            logging.debug('overriding SAML settings from JSON')
-            dict_deep_update(data, app.config['SAML_RAW_JSON_SETTINGS'])
+        if settings.SAML_RAW_JSON_SETTINGS:
+            logger.debug('overriding SAML settings from JSON')
+            dict_deep_update(data, settings.SAML_RAW_JSON_SETTINGS)
 
-        logging.debug('Rendered SAML settings: {!r}'.format(data))
+        logger.debug('Rendered SAML settings: {!r}'.format(data))
 
         return data
 
@@ -568,12 +555,12 @@ class SamlAuthenticator(AbstractUserAuthenticator):
 
         auth = self._saml_auth()
 
-        logging.debug('Processing SAML response')
+        logger.debug('Processing SAML response')
 
         try:
             request_id = session['saml_authn_request_id']
         except KeyError:
-            logging.warning('No saml_authn_request_id in session')
+            logger.warning('No saml_authn_request_id in session')
             resp = jsonify(errors=['invalid_response'],
                            message='SAML request failed',
                            reason=('No AuthNRequest ID from SP found '
@@ -589,19 +576,19 @@ class SamlAuthenticator(AbstractUserAuthenticator):
         session.pop('saml_authn_request_id', None)
 
         if not auth.is_authenticated():
-            logging.warning('auth.is_authenticated() => False')
+            logger.warning('auth.is_authenticated() => False')
             resp = jsonify(error='Not Authenticated')
             resp.status_code = 401
             return resp
 
         nameid = auth.get_nameid()
-        logging.info('SAML user authenticated: {!r}'.format(nameid))
+        logger.info('SAML user authenticated: {!r}'.format(nameid))
 
         attributes = auth.get_attributes()
-        logging.info('SAML attributes: {!r}'.format(attributes))
+        logger.info('SAML attributes: {!r}'.format(attributes))
 
         # normalize attributes by flattening single-item arrays
-        for key, val in attributes.iteritems():
+        for key, val in attributes.items():
             if isinstance(val, list) and len(val) == 1:
                 attributes[key] = val[0]
 
@@ -617,9 +604,9 @@ class SamlAuthenticator(AbstractUserAuthenticator):
         kwargs['email'] = attributes.get('email', nameid)
 
         # use first_name, last_name if present
-        for key, val in attributes.iteritems():
+        for key, val in attributes.items():
             if not getattr(key, 'lower', None):
-                logging.error('Bad list attr {!r}'.format({key: val}))
+                logger.error('Bad list attr {!r}'.format({key: val}))
             if key.lower() in ['firstname', 'first_name']:
                 kwargs['first_name'] = val
             if key.lower() in ['lastname', 'last_name']:
@@ -629,7 +616,7 @@ class SamlAuthenticator(AbstractUserAuthenticator):
         self.set_current_user(**kwargs)
 
         # success, redirect to RelayState if present or to /
-        default_redirect = flask.url_for('index')
+        default_redirect = flask.url_for('static_files.index')
         redirect_url = request.form.get('RelayState', default_redirect)
 
         # avoid redirect loop
@@ -639,7 +626,7 @@ class SamlAuthenticator(AbstractUserAuthenticator):
                 redirect_url.endswith('/login')):
             redirect_url = default_redirect
 
-        logging.debug("Redirecting to {0}".format(redirect_url))
+        logger.debug("Redirecting to {0}".format(redirect_url))
         resp = flask.redirect(redirect_url)
         self.set_csrf_token(resp)
         return resp
@@ -649,14 +636,14 @@ class SamlAuthenticator(AbstractUserAuthenticator):
         Initiate SAML SLO redirect.
         """
 
-        logging.info('Initiating SAML logout request')
+        logger.info('Initiating SAML logout request')
 
         try:
             current_nameid = self._current_user_nameid()
             current_session_id = self._current_saml_session_id()
         except errors.UserUnknownError:
             # must be already logged out
-            logging.warning('No SAML data in session. Cannot SLO log out')
+            logger.warning('No SAML data in session. Cannot SLO log out')
             self.clear_session()
             return self.redirect_to_goodbye()
 
@@ -664,7 +651,7 @@ class SamlAuthenticator(AbstractUserAuthenticator):
 
         # check for SLO support
         if not auth.get_slo_url():
-            logging.warning('No SingleLogOut endpoint defined for IdP')
+            logger.warning('No SingleLogOut endpoint defined for IdP')
             self.clear_session()
             return self.redirect_to_goodbye()
 
@@ -685,7 +672,7 @@ class SamlAuthenticator(AbstractUserAuthenticator):
         On failure, renders error JSON. On success, redirects to /goodbye.
         """
 
-        logging.debug('Processing SAML logout response')
+        logger.debug('Processing SAML logout response')
 
         auth = self._saml_auth()
         errors = []
@@ -698,7 +685,7 @@ class SamlAuthenticator(AbstractUserAuthenticator):
 
             return self._render_saml_errors_json(auth)
 
-        logging.info('SAML SLO request was successful')
+        logger.info('SAML SLO request was successful')
         self.clear_session()
 
         return self.redirect_to_goodbye()
@@ -734,11 +721,11 @@ class SamlAuthenticator(AbstractUserAuthenticator):
         if flask_request is None:
             flask_request = flask.request
 
-        url_data = urlparse.urlparse(flask_request.url)
+        url_data = urlparse(flask_request.url)
 
         if flask_request.scheme == 'https':
             https = 'on'
-        elif app.debug and app.config['SAML_FAKE_HTTPS']:
+        elif current_app.debug and settings.SAML_FAKE_HTTPS:
             https = 'on'
         else:
             https = 'off'
@@ -795,14 +782,14 @@ class SamlAuthenticator(AbstractUserAuthenticator):
         :rtype: flask.Response
         """
 
-        logging.warn('Handling SAML errors')
+        logger.warning('Handling SAML errors')
         data = {
             'message': 'SAML request failed',
             'errors': auth.get_errors(),
             'reason': auth.get_last_error_reason(),
             'request_id': auth.get_last_request_id(),
         }
-        logging.warn('Errors: {0}'.format(data))
+        logger.warning('Errors: {0}'.format(data))
 
         resp = jsonify(**data)
         resp.status_code = 500
