@@ -1,5 +1,5 @@
-from confidant.app import app
+from confidant import settings
 from confidant.utils.dynamodb import create_dynamodb_tables
 
-if app.config['DYNAMODB_CREATE_TABLE']:
+if settings.DYNAMODB_CREATE_TABLE:
     create_dynamodb_tables()
