@@ -1,12 +1,11 @@
-import base64
 import json
-import yaml
 import logging
 from os import getenv
+from base64 import b64decode
 
-from cryptography.fernet import Fernet
+from confidant.encrypted_settings import EncryptedSettings
 
-from confidant.lib import cryptolib
+logger = logging.getLogger(__name__)
 
 
 class SettingsError(Exception):
@@ -64,36 +63,6 @@ def str_env(var_name, default=''):
     return getenv(var_name, default)
 
 
-def _bootstrap(secrets):
-    """
-    Decrypt secrets and return a dict of secrets.
-    """
-    if not secrets:
-        logging.info('SECRETS_BOOTSTRAP not set, skipping bootstrapping.')
-        return {}
-    if secrets.startswith('file://'):
-        try:
-            with open(secrets[7:], 'r') as f:
-                _secrets = json.load(f)
-        except IOError:
-            logging.error(
-                'Failed to load file specified in SECRETS_BOOTSTRAP.'
-            )
-            return {}
-    else:
-        _secrets = json.loads(secrets)
-    key = cryptolib.decrypt_datakey(
-        base64.b64decode(_secrets['data_key']),
-        {'type': 'bootstrap'}
-    )
-    f = Fernet(key)
-    decrypted_secrets = yaml.safe_load(
-        f.decrypt(_secrets['secrets'].encode('utf-8'))
-    )
-    logging.info('Loaded SECRETS_BOOTSTRAP.')
-    return decrypted_secrets
-
-
 # Basic setup
 
 # Whether or not Confidant is run in debug mode. Never run confidant in debug
@@ -107,6 +76,9 @@ PORT = int_env('PORT', 8080)
 # to 'dist'.
 STATIC_FOLDER = str_env('STATIC_FOLDER', 'public')
 
+# A custom endpoint url for KMS, for use in development
+KMS_URL = str_env('KMS_URL', None)
+
 # Bootstrapping
 
 # A base64 encoded and KMS encrypted YAML string that contains secrets that
@@ -116,8 +88,7 @@ STATIC_FOLDER = str_env('STATIC_FOLDER', 'public')
 # If SECRETS_BOOTSTRAP starts with file://, then it will load the blob from a
 # file, rather than reading the blob from the environment.
 SECRETS_BOOTSTRAP = str_env('SECRETS_BOOTSTRAP')
-
-_secrets_bootstrap = _bootstrap(SECRETS_BOOTSTRAP)
+encrypted_settings = EncryptedSettings(SECRETS_BOOTSTRAP, KMS_URL)
 
 # User authentication method switcher.
 # Supported methods:
@@ -170,13 +141,13 @@ SAML_SP_CERT = str_env('SAML_SP_CERT')
 SAML_SP_KEY_FILE = str_env('SAML_SP_KEY_FILE')
 # Password for the SAML_SP_KEY_FILE
 # This setting can be loaded from the SECRETS_BOOTSTRAP.
-SAML_SP_KEY_FILE_PASSWORD = _secrets_bootstrap.get(
+SAML_SP_KEY_FILE_PASSWORD = encrypted_settings.register(
     'SAML_SP_KEY_FILE_PASSWORD',
     str_env('SAML_SP_KEY_FILE_PASSWORD', None)
 )
 # Raw SP private key in PEM format
 # This setting can be loaded from the SECRETS_BOOTSTRAP.
-SAML_SP_KEY = _secrets_bootstrap.get(
+SAML_SP_KEY = encrypted_settings.register(
     'SAML_SP_KEY',
     str_env('SAML_SP_KEY')
 )
@@ -222,19 +193,19 @@ SAML_RAW_JSON_SETTINGS = json.loads(str_env('SAML_RAW_JSON_SETTINGS', 'null'))
 
 # The client ID provided by Google's developer console.
 # This setting can be loaded from the SECRETS_BOOTSTRAP.
-GOOGLE_OAUTH_CLIENT_ID = _secrets_bootstrap.get(
+GOOGLE_OAUTH_CLIENT_ID = encrypted_settings.register(
     'GOOGLE_OAUTH_CLIENT_ID',
     str_env('GOOGLE_OAUTH_CLIENT_ID')
 )
 # The consumer secret provided by Google's developer console.
 # This setting can be loaded from the SECRETS_BOOTSTRAP.
-GOOGLE_OAUTH_CONSUMER_SECRET = _secrets_bootstrap.get(
+GOOGLE_OAUTH_CONSUMER_SECRET = encrypted_settings.register(
     'GOOGLE_OAUTH_CONSUMER_SECRET',
     str_env('GOOGLE_OAUTH_CONSUMER_SECRET')
 )
 # A randomly generated string that can be used as a salt for the OAuth2 flow.
 # This setting can be loaded from the SECRETS_BOOTSTRAP.
-AUTHOMATIC_SALT = _secrets_bootstrap.get(
+AUTHOMATIC_SALT = encrypted_settings.register(
     'AUTHOMATIC_SALT',
     str_env('AUTHOMATIC_SALT')
 )
@@ -252,7 +223,6 @@ HEADER_AUTH_FIRST_NAME_HEADER = str_env('HEADER_AUTH_FIRST_NAME_HEADER')
 # The name of the header that will contain the user's last name.  Optional.
 HEADER_AUTH_LAST_NAME_HEADER = str_env('HEADER_AUTH_LAST_NAME_HEADER')
 
-
 # KMS service authentication
 
 # The 'to' context used in KMS auth. This should be set to the name of the IAM
@@ -262,7 +232,8 @@ AUTH_CONTEXT = str_env('AUTH_CONTEXT')
 # as KMS_MASTER_KEY, but it's highly recommended to use a different key for
 # authentication and at-rest encryption. This key is specifically for the
 # 'service' role.
-# Example: mykmskey
+# If a key alias is used, rather than an ARN, it must be prefixed with: alias/
+# Example: alias/mykmskey
 AUTH_KEY = str_env('AUTH_KEY')
 # A dict of KMS key to account mappings. These keys are for the 'service' role
 # to support multiple AWS accounts. If services are scoped to accounts,
@@ -274,6 +245,8 @@ SCOPED_AUTH_KEYS = json.loads(str_env('SCOPED_AUTH_KEYS', '{}'))
 # The alias of the KMS key being used for authentication that is specifically
 # for the 'user' role. This should not be the same key as AUTH_KEY if your
 # kms token version is < 2, as it would allow services to masquerade as users.
+# If a key alias is used, rather than an ARN, it must be prefixed with: alias/
+# Example: alias/mykmskey
 USER_AUTH_KEY = str_env('USER_AUTH_KEY')
 # The maximum lifetime of an authentication token in minutes.
 AUTH_TOKEN_MAX_LIFETIME = int_env('AUTH_TOKEN_MAX_LIFETIME', 60)
@@ -289,6 +262,8 @@ KMS_AUTH_MANAGE_GRANTS = bool_env('KMS_AUTH_MANAGE_GRANTS', True)
 # equivalent to the number of tokens you expect to generate within the lifetime
 # of your tokens.
 KMS_AUTH_TOKEN_CACHE_SIZE = int_env('KMS_AUTH_TOKEN_CACHE_SIZE', 4096)
+# See also KMS_URL, defined above, for use in the EncryptedSettings
+# initialization, which can be used to point boto at a local KMS.
 
 # SSL redirection and HSTS
 
@@ -307,9 +282,9 @@ XSRF_COOKIE_NAME = str_env('XSRF_COOKIE_NAME', 'XSRF-TOKEN')
 # Session cache
 # Mutually exclusive with secure cookie session settings.
 
-# A redis connection url.
+# A redis connection url to store flask sessions.
 # Example: redis://localhost:6379
-REDIS_URL = str_env('REDIS_URL')
+REDIS_URL_FLASK_SESSIONS = str_env('REDIS_URL_FLASK_SESSIONS')
 # The session type for Flask-Session. Currenty only redis is supported.
 SESSION_TYPE = str_env('SESSION_TYPE', 'redis')
 # The key prefix to use in redis. Can be used to run multiple applications on
@@ -321,7 +296,7 @@ SESSION_KEY_PREFIX = str_env('SESSION_KEY_PREFIX', 'confidant:')
 SESSION_USE_SIGNER = bool_env('SESSION_USE_SIGNER', True)
 # A long randomly generated string.
 # This setting can be loaded from the SECRETS_BOOTSTRAP.
-SESSION_SECRET = _secrets_bootstrap.get(
+SESSION_SECRET = encrypted_settings.register(
     'SESSION_SECRET',
     str_env('SESSION_SECRET')
 )
@@ -357,17 +332,27 @@ DYNAMODB_URL = str_env('DYNAMODB_URL')
 # The DynamoDB table to use for storage.
 # Example: mydynamodbtable
 DYNAMODB_TABLE = str_env('DYNAMODB_TABLE')
+# The DynamoDB table to use for permanently archiving old credentials and
+# services.
+# Example: mydynamodbtable-archive
+DYNAMODB_TABLE_ARCHIVE = str_env('DYNAMODB_TABLE_ARCHIVE')
 # Have PynamoDB automatically generate the DynamoDB table if it doesn't exist.
 # Note that you need to give Confidant's IAM user or role enough privileges for
 # this to occur.
 DYNAMODB_CREATE_TABLE = bool_env('DYNAMODB_CREATE_TABLE', False)
 # Connection pool size for PynamoDB connections to DynamoDB
 PYNAMO_CONNECTION_POOL_SIZE = int_env('PYNAMO_CONNECTION_POOL_SIZE', 100)
-PYNAMO_REQUEST_TIMEOUT_SECONDS = int_env('PYNAMO_REQUEST_TIMEOUT_SECONDS', 1)
+PYNAMO_CONNECT_TIMEOUT_SECONDS = int_env('PYNAMO_CONNECT_TIMEOUT_SECONDS', 1)
+PYNAMO_READ_TIMEOUT_SECONDS = int_env('PYNAMO_READ_TIMEOUT_SECONDS', 1)
+# page limit size for history API endpoints listing
+HISTORY_PAGE_LIMIT = int_env('HISTORY_PAGE_LIMIT')
+if HISTORY_PAGE_LIMIT == 0:
+    HISTORY_PAGE_LIMIT = None
 
 # Encryption
 
 # The KMS key to use for at-rest encryption for secrets in DynamoDB.
+# If a key alias is used, rather than an ARN, it must be prefixed with: alias/
 KMS_MASTER_KEY = str_env('KMS_MASTER_KEY')
 
 # Graphite events
@@ -378,14 +363,14 @@ GRAPHITE_EVENT_URL = str_env('GRAPHITE_EVENT_URL')
 # A basic auth username.
 # Example: mygraphiteuser
 # This setting can be loaded from the SECRETS_BOOTSTRAP.
-GRAPHITE_USERNAME = _secrets_bootstrap.get(
+GRAPHITE_USERNAME = encrypted_settings.register(
     'GRAPHITE_USERNAME',
     str_env('GRAPHITE_USERNAME')
 )
 # A basic auth password:
 # Example: mylongandsupersecuregraphitepassword
 # This setting can be loaded from the SECRETS_BOOTSTRAP.
-GRAPHITE_PASSWORD = _secrets_bootstrap.get(
+GRAPHITE_PASSWORD = encrypted_settings.register(
     'GRAPHITE_PASSWORD',
     str_env('GRAPHITE_PASSWORD')
 )
@@ -404,14 +389,14 @@ WEBHOOK_URL = str_env('WEBHOOK_URL')
 # A basic auth username.
 # Example: myhookuser
 # This setting can be loaded from the SECRETS_BOOTSTRAP.
-WEBHOOK_USERNAME = _secrets_bootstrap.get(
+WEBHOOK_USERNAME = encrypted_settings.register(
     'WEBHOOK_USERNAME',
     str_env('WEBHOOK_USERNAME')
 )
 # A basic auth password:
 # Example: mylongandsupersecurehookpassword
 # This setting can be loaded from the SECRETS_BOOTSTRAP.
-WEBHOOK_PASSWORD = _secrets_bootstrap.get(
+WEBHOOK_PASSWORD = encrypted_settings.register(
     'WEBHOOK_PASSWORD',
     str_env('WEBHOOK_PASSWORD')
 )
@@ -421,7 +406,6 @@ WEBHOOK_PASSWORD = _secrets_bootstrap.get(
 # in different credentials associated with a service.
 IGNORE_CONFLICTS = bool_env('IGNORE_CONFLICTS', False)
 
-
 # Customization
 
 # Directory for customization of AngularJS frontend.
@@ -443,6 +427,21 @@ CUSTOM_FRONTEND_DIRECTORY = str_env('CUSTOM_FRONTEND_DIRECTORY')
 # }
 CLIENT_CONFIG = json.loads(str_env('CLIENT_CONFIG', '{}'))
 
+# Maintenance mode
+
+# Maintenance mode allows you to disable writes from the API, making it
+# possible to do migrations or other maintenance actions without worrying
+# about writes interfering with your actions. Maintenance mode can be enabled
+# either through the MAINTENANCE_MODE_ENABLED configuration option, or through
+# a touch file specified via the MAINTENANCE_MODE_TOUCH_FILE configuration
+# option.
+MAINTENANCE_MODE = bool_env('MAINTENANCE_MODE', False)
+MAINTENANCE_MODE_TOUCH_FILE = str_env('MAINTENANCE_MODE_TOUCH_FILE')
+
+# Enforce users to add documentation to their credentials on how to rotate
+# them, for easier rotation in the case a credential is expired or compromised.
+ENFORCE_DOCUMENTATION = bool_env('ENFORCE_DOCUMENTATION', False)
+
 # Test/Development
 
 # Whether or not authentication is required. Unless doing testing or
@@ -480,10 +479,253 @@ AWS_DEFAULT_REGION = str_env('AWS_DEFAULT_REGION', 'us-east-1')
 #
 # GEVENT_RESOLVER='ares'
 
+# IAM role cache configuration
+
+# Whether or not we keep a hot in-process cache of IAM roles, refreshed by a
+# gevent thread.
+BACKGROUND_CACHE_IAM_ROLES = bool_env('BACKGROUND_CACHE_IAM_ROLES', True)
+# Number of seconds between calls to refresh the IAM role cache. Calls will be
+# randomized +/- by BACKGROUND_CACHE_IAM_ROLE_JITTER seconds,
+# to randomize calls across processes. Minimum value for
+# this setting is 60.
+BACKGROUND_CACHE_IAM_ROLE_REFRESH_RATE = int_env(
+    'BACKGROUND_CACHE_IAM_ROLE_REFRESH_RATE',
+    600
+)
+
+# Seconds to add as jitter to ensure all processes do not refresh at
+# the same time which can cause AWS ratelimits to be hit.
+# Default to 20 seconds
+BACKGROUND_CACHE_IAM_ROLE_JITTER = int_env(
+    'BACKGROUND_CACHE_IAM_ROLE_JITTER',
+    20
+)
+
+# ACM Private CA configuration
+
+# ACM_PRIVATE_CAS is a comma separated list of friendly ca names. Confidant
+# will use these names to load settings for each CA, by appending the name
+# to the sub-setting. e.g.:
+#   ACM_PRIVATE_CAS=testca1,testca2
+#   ACM_PRIVATE_CA_ARN_TESTCA1=arn:...
+#   ACM_PRIVATE_CA_ARN_TESTCA2=arn:...
+ACM_PRIVATE_CAS = str_env('ACM_PRIVATE_CAS').split(',')
+# ACM_PRIVATE_CA_SETTINGS is used internally, and can not be set via
+# environment.
+ACM_PRIVATE_CA_SETTINGS = {}
+for ca in ACM_PRIVATE_CAS:
+    # Skip any empty values (such as ACM_PRIVATE_CAS being unset)
+    if not ca:
+        continue
+    # Folks may set the list to "a, b, c" so we need to strip whitespace
+    ca = ca.strip()
+    # We require environment to be all uppercase, including the CA name.
+    ca_up = ca.upper()
+    ca_settings = {}
+    # The full ARN of the private CA
+    ca_settings['arn'] = str_env('ACM_PRIVATE_CA_ARN_{}'.format(ca_up))
+    # The signing algorithm to use when calling issue certificate in ACM
+    ca_settings['signing_algorithm'] = str_env(
+        'ACM_PRIVATE_CA_SIGNING_ALGORITHM_{}'.format(ca_up),
+        'SHA256WITHRSA',
+    )
+    # The full ARN of the certificate template used when issuing certificates.
+    # The default value used here allows for client and server authentication.
+    ca_settings['template_arn'] = str_env(
+        'ACM_PRIVATE_CA_TEMPLATE_ARN_{}'.format(ca_up),
+        'arn:aws:acm-pca:::template/EndEntityCertificate/V1',
+    )
+    # Maximum validity of certificates issued from the private CA. If clients
+    # request a higher validity period, we'll set the validity to this value.
+    ca_settings['max_validity_days'] = int_env(
+        'ACM_PRIVATE_CA_MAX_VALIDITY_DAYS_{}'.format(ca_up),
+        120,
+    )
+    # Attributes to use when generating CSRs
+    ca_settings['csr_country_name'] = str_env(
+        'ACM_PRIVATE_CA_CSR_COUNTRY_NAME_{}'.format(ca_up),
+    )
+    ca_settings['csr_state_or_province_name'] = str_env(
+        'ACM_PRIVATE_CA_CSR_STATE_OR_PROVINCE_NAME_{}'.format(ca_up),
+    )
+    ca_settings['csr_locality_name'] = str_env(
+        'ACM_PRIVATE_CA_CSR_LOCALITY_NAME_{}'.format(ca_up),
+    )
+    ca_settings['csr_organization_name'] = str_env(
+        'ACM_PRIVATE_CA_CSR_ORGANIZATION_NAME_{}'.format(ca_up),
+    )
+    # Whether we generate self-signed certificates, or issue certificates from
+    # the private CA. This is intended for testing and development purposes.
+    ca_settings['self_sign'] = bool_env(
+        'ACM_PRIVATE_CA_SELF_SIGN_{}'.format(ca_up),
+        False,
+    )
+    # Size of private keys generated.
+    ca_settings['key_size'] = int_env(
+        'ACM_PRIVATE_CA_KEY_SIZE_{}'.format(ca_up),
+        2048,
+    )
+    # Exponent size used when generating keys. You probably don't want to change
+    # this.
+    ca_settings['key_public_exponent_size'] = int_env(
+        'ACM_PRIVATE_CA_KEY_PUBLIC_EXPONENT_SIZE_{}'.format(ca_up),
+        65537,
+    )
+    # Whether or not to cache certificates, when generating key/csr/certificates
+    # and calling into ACM private CA to issue certificates.
+    ca_settings['certificate_use_cache'] = bool_env(
+        'ACM_PRIVATE_CA_CERTIFICATE_USE_CACHE_{}'.format(ca_up),
+        False,
+    )
+    # Number of certificates to cache, when generating key/csr/certificates and
+    # calling into ACM private CA to issue certificates. This should be high
+    # enough to cache all concurrent certificates being issued at one time.
+    ca_settings['certificate_cache_size'] = int_env(
+        'ACM_PRIVATE_CA_CERTIFICATE_CACHE_SIZE_{}'.format(ca_up),
+        1028,
+    )
+    # A regex to match against CN and SAN values for this CA. This regex must
+    # include a named group for service_name: (?P<service_name>)
+    # If no named group is defined, then the default ACL will deny generation
+    # of the certificate.
+    # Any certificate issue attempt not matching this pattern for CN or values
+    # in SAN will be denied. If this is unset, all certificate issue attempts
+    # will be denied by the default_acl.
+    #     Example: (?P<service_name>[\w-]+)\.example\.com
+    #     Example match: test-service.example.com
+    #     service_name from example: test-service
+    ca_settings['name_regex'] = str_env(
+        'ACM_PRIVATE_CA_DOMAIN_REGEX_{}'.format(ca_up),
+    )
+    ACM_PRIVATE_CA_SETTINGS[ca] = ca_settings
+
+
+MAXIMUM_ROTATION_DAYS = int_env('MAXIMUM_ROTATION_DAYS')
+# Credentials can be "tagged" (eg: FINANCIALLY_SENSITIVE or ADMIN_PRIV)
+# Certain tags might never need to be rotated
+TAGS_EXCLUDING_ROTATION = json.loads(str_env('TAGS_EXCLUDING_ROTATION', '[]'))
+# Credentials with different tags might have different rotation schedules
+# We use this config to specify how many days each type of credential should
+# be rotated
+ROTATION_DAYS_CONFIG = json.loads(str_env('ROTATION_DAYS_CONFIG', '{}'))
+
+# If this is eanbled, update credential.last_decrypted_date
+# when credential.credential_pairs is sent back to the client
+# in GET /v1/credentials/<ID> to keep track of when a human
+# last saw a credential pair
+ENABLE_SAVE_LAST_DECRYPTION_TIME = bool_env('ENABLE_SAVE_LAST_DECRYPTION_TIME')
+
+# Add any certificate authorities
+# Should be in encrypted settings following this format (where name is
+# the name of the environment) and key ids must be unique:
+# [
+# {
+#   "key": "--- RSA...",
+#   "crt": "--- CERT...",
+#   "name": "staging",
+#   "passphrase": "some-key",
+#   "kid": "some-kid"
+# }, ...
+# ]
+
+if bool_env("JWT_IS_CA_ENCRYPTED", True):
+    decrypted_jwt_cas = encrypted_settings.decrypted_secrets.get(
+        'JWT_CERTIFICATE_AUTHORITIES'
+    )
+else:
+    decrypted_jwt_cas = str_env('JWT_CERTIFICATE_AUTHORITIES')
+
+JWT_CERTIFICATE_AUTHORITIES = json.loads(b64decode(decrypted_jwt_cas)) \
+    if decrypted_jwt_cas else {}
+
+JWT_CACHING_ENABLED = bool_env('JWT_CACHING_ENABLED', False)
+
+JWT_CACHING_MAX_SIZE = int_env('JWT_CACHING_MAX_SIZE', 1000)
+
+# Maximum time JWTs are stored in Confidant's cache.
+# Warning: this needs to be considerably less than the JWT TTL
+# to avoid Confidant issuing very short lived JWTs.
+# Enabling the cache means JWTs will have a varying minimum/maximum TTL window.
+# example:
+#     JWT_CACHING_TTL_SECONDS = 900 (15 min)
+#     JWT_DEFAULT_JWT_EXPIRATION_SECONDS = 3600 (1 hr)
+# This means JWTs issued will be between 3600 and 2700 seconds
+JWT_CACHING_TTL_SECONDS = int_env('JWT_CACHING_TTL_SECONDS', 900)
+
+JWT_DEFAULT_JWT_EXPIRATION_SECONDS = int_env(
+    'JWT_DEFAULT_JWT_EXPIRATION_SECONDS', 3600
+)
+
+# A redis connection url to store JWTs.
+# Example: redis://localhost:6379
+REDIS_URL_JWT_CACHE = str_env('REDIS_URL_JWT_CACHE', '')
+# Redis socket timeout in seconds
+REDIS_SOCKET_TIMEOUT = int_env('REDIS_SOCKET_TIMEOUT', 2)
+# Setting it true uses REDIS for JWT cache
+JWT_CACHING_USE_REDIS = bool_env('JWT_CACHING_USE_REDIS', False)
+
+
+# Key IDs from CERTIFICATE_AUTHORITIES that should be used to sign new JWTs,
+# provide a JSON with the following format:
+# {"staging": "some_kid", "production": "some_kid"}
+JWT_ACTIVE_SIGNING_KEYS = json.loads(str_env('JWT_ACTIVE_SIGNING_KEYS', '{}'))
+
+# CUSTOM_CA_ENCRYPTED denotes whether provided CUSTOM_CERTIFICATE_AUTHORITIES
+# is encrypted or not. If it is encrypted, it will be decrypted before use.
+# It should be encrypted for non-development environments.
+if bool_env('CUSTOM_CA_ENCRYPTED', True):
+    decrypted_custom_cas = encrypted_settings.decrypted_secrets.get(
+        'CUSTOM_CERTIFICATE_AUTHORITIES'
+    )
+else:
+    decrypted_custom_cas = str_env('CUSTOM_CERTIFICATE_AUTHORITIES')
+
+# CA_TYPE for issuing certificates, defaults to aws_acm_pca.
+# options: aws_acm_pca, custom
+CA_TYPE = str_env('CA_TYPE', "aws_acm_pca")
+
+# CUSTOM_CERTIFICATE_AUTHORITIES
+# Should be in encrypted settings following this
+# format (where name is the name of the environment) and key ids must be unique:
+# {"<name>":[{
+#   "key": "--- RSA...",
+#   "rootcrt": "--- CERT...",
+#   "crt": "--- CERT...",
+#   "passphrase": "some-key",
+#   "kid": "some-kid"
+# }, ...
+# ]}
+CUSTOM_CERTIFICATE_AUTHORITIES = json.loads(b64decode(decrypted_custom_cas)) \
+    if decrypted_custom_cas else {}
+
+# provide a JSON with the following format:
+# {"staging": "some_kid", "production": "some_kid"}
+CUSTOM_CA_ACTIVE_KEYS = json.loads(str_env('CUSTOM_CA_ACTIVE_KEYS', '{}'))
+
+CUSTOM_CA_SETTINGS = {}
+CUSTOM_CA_SETTINGS["max_validity_days"] = int_env(
+    "CUSTOM_CA_MAX_VALIDITY_DAYS",
+    120,
+)
+# A regex to match against CN and SAN values for this CA. This regex must
+# include a named group for service_name: (?P<service_name>)
+# If no named group is defined, then the default ACL will deny generation
+# of the certificate.
+# Any certificate issue attempt not matching this pattern for CN or values
+# in SAN will be denied. If this is unset, all certificate issue attempts
+# will be denied by the default_acl.
+#     Example: (?P<service_name>[\w-]+)\.example\.com
+#     Example match: test-service.example.com
+#     service_name from example: test-service
+CUSTOM_CA_SETTINGS["name_regex"] = str_env(
+    "CUSTOM_CA_DOMAIN_REGEX",
+    None,
+)
+
 # Configuration validation
 _settings_failures = False
-if len(list(set(SCOPED_AUTH_KEYS.values()))) != len(SCOPED_AUTH_KEYS.values()):
-    logging.error('SCOPED_AUTH_KEYS values are not unique.')
+if len(set(SCOPED_AUTH_KEYS.values())) != len(SCOPED_AUTH_KEYS.values()):
+    logger.error('SCOPED_AUTH_KEYS values are not unique.')
     _settings_failures = True
 
 if _settings_failures:
@@ -494,4 +736,10 @@ def get(name, default=None):
     """
     Get the value of a variable in the settings module scope.
     """
+    if encrypted_settings.registered(name):
+        return encrypted_settings.get_secret(name)
     return globals().get(name, default)
+
+
+# Module that will perform an external ACL check on API endpoints
+ACL_MODULE = str_env('ACL_MODULE', 'confidant.authnz.rbac:default_acl')
